from concurrent.futures import Future
from enum import IntEnum
from threading import Event
from typing import Callable, List, Optional, Tuple

from awscrt import NativeResource as NativeResource
from awscrt.auth import AwsCredentialsProvider as AwsCredentialsProvider
from awscrt.http import HttpRequest as HttpRequest
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import TlsConnectionOptions as TlsConnectionOptions

class S3RequestType(IntEnum):
    DEFAULT: int
    GET_OBJECT: int
    PUT_OBJECT: int

class S3RequestTlsMode(IntEnum):
    ENABLED: int
    DISABLED: int

class S3Client(NativeResource):
    shutdown_event: Event
    def __init__(
        self,
        *,
        bootstrap: Optional[ClientBootstrap],
        region: str,
        tls_mode: Optional[S3RequestTlsMode] = ...,
        credential_provider: Optional[S3RequestTlsMode] = ...,
        tls_connection_options: Optional[TlsConnectionOptions] = ...,
        part_size: Optional[int] = ...,
        throughput_target_gbps: Optional[float] = ...,
    ) -> None: ...
    def make_request(
        self,
        *,
        request: HttpRequest,
        type: S3RequestType,
        credential_provider: Optional[AwsCredentialsProvider] = ...,
        recv_filepath: Optional[str] = ...,
        send_filepath: Optional[str] = ...,
        on_headers: Optional[Callable[[int, List[Tuple[str, str]]], None]] = ...,
        on_body: Optional[Callable[[bytes, int], None]] = ...,
        on_done: Optional[
            Callable[
                [Optional[BaseException], Optional[List[Tuple[str, str]]], Optional[bytes]], None
            ]
        ] = ...,
        on_progress: Optional[Callable[[int], None]] = ...,
    ) -> S3Request: ...

class S3Request(NativeResource):
    shutdown_event: Event
    def __init__(
        self,
        *,
        client: S3Client,
        request: HttpRequest,
        type: S3RequestType,
        credential_provider: Optional[AwsCredentialsProvider] = ...,
        recv_filepath: Optional[str] = ...,
        send_filepath: Optional[str] = ...,
        on_headers: Optional[Callable[[int, List[Tuple[str, str]]], None]] = ...,
        on_body: Optional[Callable[[bytes, int], None]] = ...,
        on_done: Optional[
            Callable[
                [Optional[BaseException], Optional[List[Tuple[str, str]]], Optional[bytes]], None
            ]
        ] = ...,
        on_progress: Optional[Callable[[int], None]] = ...,
        region: Optional[str] = ...,
    ) -> None: ...
    @property
    def finished_future(self) -> Future[Optional[BaseException]]: ...
    def cancel(self) -> None: ...

class _S3ClientCore:
    def __init__(
        self,
        bootstrap: ClientBootstrap,
        credential_provider: Optional[AwsCredentialsProvider] = ...,
        tls_connection_options: Optional[TlsConnectionOptions] = ...,
    ) -> None: ...

class _S3RequestCore:
    def __init__(
        self,
        request: HttpRequest,
        finish_future: Future[Optional[BaseException]],
        shutdown_event: Event,
        credential_provider: Optional[AwsCredentialsProvider] = ...,
        on_headers: Optional[Callable[[int, List[Tuple[str, str]]], None]] = ...,
        on_body: Optional[Callable[[bytes, int], None]] = ...,
        on_done: Optional[
            Callable[
                [Optional[BaseException], Optional[List[Tuple[str, str]]], Optional[bytes]], None
            ]
        ] = ...,
        on_progress: Optional[Callable[[int], None]] = ...,
    ) -> None: ...



from awscrt import NativeResource as NativeResource
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import DefaultHostResolver as DefaultHostResolver
from awscrt.io import EventLoopGroup as EventLoopGroup

def native_memory_usage() -> int: ...
def dump_native_memory() -> None: ...
def join_all_native_threads(*, timeout_sec: float = ...) -> bool: ...
def check_for_leaks(*, timeout_sec: float = ...) -> None: ...



from dataclasses import dataclass
from enum import IntEnum
from typing import Any, Callable, Optional, Sequence, Tuple, Union

from awscrt import NativeResource as NativeResource
from awscrt.http import HttpProxyOptions as HttpProxyOptions
from awscrt.http import HttpRequest as HttpRequest
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import SocketOptions as SocketOptions
from awscrt.io import TlsConnectionOptions as TlsConnectionOptions

class Opcode(IntEnum):
    CONTINUATION: int
    TEXT: int
    BINARY: int
    CLOSE: int
    PING: int
    PONG: int
    def is_data_frame(self) -> bool: ...

MAX_PAYLOAD_LENGTH: int

@dataclass
class OnConnectionSetupData:
    exception: Optional[Exception] = ...
    websocket: Optional["WebSocket"] = ...
    handshake_response_status: Optional[int] = ...
    handshake_response_headers: Optional[Sequence[Tuple[str, str]]] = ...
    handshake_response_body: Optional[bytes] = ...

@dataclass
class OnConnectionShutdownData:
    exception: Optional[Exception] = ...

@dataclass
class IncomingFrame:
    opcode: Opcode
    payload_length: int
    fin: bool
    def is_data_frame(self) -> bool: ...

@dataclass
class OnIncomingFrameBeginData:
    frame: IncomingFrame

@dataclass
class OnIncomingFramePayloadData:
    frame: IncomingFrame
    data: bytes

@dataclass
class OnIncomingFrameCompleteData:
    frame: IncomingFrame
    exception: Optional[Exception] = ...

@dataclass
class OnSendFrameCompleteData:
    exception: Optional[Exception] = ...

class WebSocket(NativeResource):
    def __init__(self, binding: Any) -> None: ...
    def close(self) -> None: ...
    def send_frame(
        self,
        opcode: Opcode,
        payload: Optional[Union[str, bytes, bytearray, memoryview]] = ...,
        *,
        fin: bool = ...,
        on_complete: Optional[Callable[[OnSendFrameCompleteData], None]] = ...,
    ) -> None: ...
    def increment_read_window(self, size: int) -> None: ...

class _WebSocketCore(NativeResource):
    def __init__(
        self,
        on_connection_setup: Callable[[OnConnectionSetupData], None],
        on_connection_shutdown: Optional[Callable[[OnConnectionShutdownData], None]],
        on_incoming_frame_begin: Optional[Callable[[OnIncomingFrameBeginData], None]],
        on_incoming_frame_payload: Optional[Callable[[OnIncomingFramePayloadData], None]],
        on_incoming_frame_complete: Optional[Callable[[OnIncomingFrameCompleteData], None]],
    ) -> None: ...

def connect(
    *,
    host: str,
    port: Optional[int] = ...,
    handshake_request: HttpRequest,
    bootstrap: Optional[ClientBootstrap] = ...,
    socket_options: Optional[SocketOptions] = ...,
    tls_connection_options: Optional[TlsConnectionOptions] = ...,
    proxy_options: Optional[HttpProxyOptions] = ...,
    manage_read_window: bool = ...,
    initial_read_window: Optional[int] = ...,
    on_connection_setup: Callable[[OnConnectionSetupData], None],
    on_connection_shutdown: Optional[Callable[[OnConnectionShutdownData], None]] = ...,
    on_incoming_frame_begin: Optional[Callable[[OnIncomingFrameBeginData], None]] = ...,
    on_incoming_frame_payload: Optional[Callable[[OnIncomingFramePayloadData], None]] = ...,
    on_incoming_frame_complete: Optional[Callable[[OnIncomingFrameCompleteData], None]] = ...,
) -> None: ...
def create_handshake_request(*, host: str, path: str = ...) -> HttpRequest: ...



from concurrent.futures import Future
from enum import IntEnum
from typing import IO, Any, Callable, Iterator, List, Optional, Tuple, Type, TypeVar

from awscrt import NativeResource as NativeResource
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import InputStream as InputStream
from awscrt.io import SocketOptions as SocketOptions
from awscrt.io import TlsConnectionOptions as TlsConnectionOptions

_R = TypeVar("_R")

class HttpVersion(IntEnum):
    Unknown: int
    Http1_0: int
    Http1_1: int
    Http2: int

class HttpConnectionBase(NativeResource):
    def __init__(self) -> None: ...
    @property
    def shutdown_future(self) -> Future[None]: ...
    @property
    def version(self) -> str: ...
    def close(self) -> Future[None]: ...
    def is_open(self) -> bool: ...

class HttpClientConnection(HttpConnectionBase):
    @classmethod
    def new(
        cls: Type[_R],
        host_name: str,
        port: int,
        bootstrap: Optional[ClientBootstrap] = ...,
        socket_options: Optional[SocketOptions] = ...,
        tls_connection_options: Optional[TlsConnectionOptions] = ...,
        proxy_options: Optional[HttpProxyOptions] = ...,
    ) -> Future[_R]: ...
    @property
    def host_name(self) -> str: ...
    @property
    def port(self) -> int: ...
    def request(
        self,
        request: HttpRequest,
        on_response: Optional[Callable[[HttpClientStream, int, List[Tuple[str, str]]], None]] = ...,
        on_body: Optional[Callable[[HttpClientStream, bytes], None]] = ...,
    ) -> HttpClientStream: ...

class HttpStreamBase(NativeResource):
    def __init__(
        self,
        connection: HttpClientConnection,
        on_body: Optional[Callable[[HttpClientStream, bytes], None]] = ...,
    ) -> None: ...
    @property
    def connection(self) -> HttpClientConnection: ...
    @property
    def completion_future(self) -> Future[int]: ...

class HttpClientStream(HttpStreamBase):
    def __init__(
        self,
        connection: HttpClientConnection,
        request: HttpRequest,
        on_response: Optional[Callable[[HttpClientStream, int, List[Tuple[str, str]]], None]] = ...,
        on_body: Optional[Callable[[HttpClientStream, bytes], None]] = ...,
    ) -> None: ...
    @property
    def response_status_code(self) -> int: ...
    def activate(self) -> None: ...

class HttpMessageBase(NativeResource):
    def __init__(
        self, binding: Any, headers: HttpHeaders, body_stream: Optional[IO[Any]] = ...
    ) -> None: ...
    @property
    def headers(self) -> HttpHeaders: ...
    @property
    def body_stream(self) -> Optional[IO[Any]]: ...
    @body_stream.setter
    def body_stream(self, stream: IO[Any]) -> None: ...

class HttpRequest(HttpMessageBase):
    def __init__(
        self,
        method: str = ...,
        path: str = ...,
        headers: Optional[HttpHeaders] = ...,
        body_stream: Optional[IO[Any]] = ...,
    ) -> None: ...
    @property
    def method(self) -> str: ...
    @method.setter
    def method(self, method: str) -> None: ...
    @property
    def path(self) -> str: ...
    @path.setter
    def path(self, path: str) -> None: ...

class HttpHeaders(NativeResource):
    def __init__(self, name_value_pairs: Optional[List[Tuple[str, str]]] = ...) -> None: ...
    def add(self, name: str, value: str) -> None: ...
    def add_pairs(self, name_value_pairs: List[Tuple[str, str]]) -> None: ...
    def set(self, name: str, value: str) -> None: ...
    def get_values(self, name: str) -> Iterator[Tuple[str, str]]: ...
    def get(self, name: str, default: Optional[str] = ...) -> Optional[str]: ...
    def remove(self, name: str) -> None: ...
    def remove_value(self, name: str, value: str) -> None: ...
    def clear(self) -> None: ...
    def __iter__(self) -> Iterator[Tuple[str, str]]: ...

class HttpProxyConnectionType(IntEnum):
    Legacy: int
    Forwarding: int
    Tunneling: int

class HttpProxyAuthenticationType(IntEnum):
    Nothing: int
    Basic: int

class HttpProxyOptions:
    def __init__(
        self,
        host_name: str,
        port: int,
        tls_connection_options: Optional[TlsConnectionOptions] = ...,
        auth_type: HttpProxyAuthenticationType = ...,
        auth_username: Optional[str] = ...,
        auth_password: Optional[str] = ...,
        connection_type: Optional[HttpProxyConnectionType] = ...,
    ) -> None:
        self.host_name: str
        self.port: int
        self.tls_connection_options: Optional[TlsConnectionOptions]
        self.auth_type: HttpProxyAuthenticationType
        self.auth_username: Optional[str]
        self.auth_password: Optional[str]
        self.connection_type: HttpProxyConnectionType



from typing import Any

class Hash:
    def __init__(self, native_handle: Any) -> None: ...
    @staticmethod
    def sha1_new() -> Hash: ...
    @staticmethod
    def sha256_new() -> Hash: ...
    @staticmethod
    def md5_new() -> Hash: ...
    def update(self, to_hash: Any) -> None: ...
    def digest(self, truncate_to: int = ...) -> str: ...

class HMAC:
    def __init__(self, native_handle: Any) -> None: ...
    @staticmethod
    def sha256_hmac_new(secret_key: str) -> HMAC: ...
    def update(self, to_hmac: Any) -> None: ...
    def digest(self, truncate_to: int = ...) -> str: ...



from collections.abc import Sequence
from concurrent.futures import Future
from dataclasses import dataclass
from enum import IntEnum
from typing import Any, Callable, Optional

from awscrt import NativeResource as NativeResource
from awscrt import exceptions as exceptions
from awscrt.http import HttpProxyOptions as HttpProxyOptions
from awscrt.http import HttpRequest as HttpRequest
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import ClientTlsContext as ClientTlsContext
from awscrt.io import SocketOptions as SocketOptions

class QoS(IntEnum):
    AT_MOST_ONCE: int
    AT_LEAST_ONCE: int
    EXACTLY_ONCE: int

class ConnectReasonCode(IntEnum):
    SUCCESS: int
    UNSPECIFIED_ERROR: int
    MALFORMED_PACKET: int
    PROTOCOL_ERROR: int
    IMPLEMENTATION_SPECIFIC_ERROR: int
    UNSUPPORTED_PROTOCOL_VERSION: int
    CLIENT_IDENTIFIER_NOT_VALID: int
    BAD_USERNAME_OR_PASSWORD: int
    NOT_AUTHORIZED: int
    SERVER_UNAVAILABLE: int
    SERVER_BUSY: int
    BANNED: int
    BAD_AUTHENTICATION_METHOD: int
    TOPIC_NAME_INVALID: int
    PACKET_TOO_LARGE: int
    QUOTA_EXCEEDED: int
    PAYLOAD_FORMAT_INVALID: int
    RETAIN_NOT_SUPPORTED: int
    QOS_NOT_SUPPORTED: int
    USE_ANOTHER_SERVER: int
    SERVER_MOVED: int
    CONNECTION_RATE_EXCEEDED: int

class DisconnectReasonCode(IntEnum):
    NORMAL_DISCONNECTION: int
    DISCONNECT_WITH_WILL_MESSAGE: int
    UNSPECIFIED_ERROR: int
    MALFORMED_PACKET: int
    PROTOCOL_ERROR: int
    IMPLEMENTATION_SPECIFIC_ERROR: int
    NOT_AUTHORIZED: int
    SERVER_BUSY: int
    SERVER_SHUTTING_DOWN: int
    KEEP_ALIVE_TIMEOUT: int
    SESSION_TAKEN_OVER: int
    TOPIC_FILTER_INVALID: int
    TOPIC_NAME_INVALID: int
    RECEIVE_MAXIMUM_EXCEEDED: int
    TOPIC_ALIAS_INVALID: int
    PACKET_TOO_LARGE: int
    MESSAGE_RATE_TOO_HIGH: int
    QUOTA_EXCEEDED: int
    ADMINISTRATIVE_ACTION: int
    PAYLOAD_FORMAT_INVALID: int
    RETAIN_NOT_SUPPORTED: int
    QOS_NOT_SUPPORTED: int
    USE_ANOTHER_SERVER: int
    SERVER_MOVED: int
    SHARED_SUBSCRIPTIONS_NOT_SUPPORTED: int
    CONNECTION_RATE_EXCEEDED: int
    MAXIMUM_CONNECT_TIME: int
    SUBSCRIPTION_IDENTIFIERS_NOT_SUPPORTED: int
    WILDCARD_SUBSCRIPTIONS_NOT_SUPPORTED: int

class PubackReasonCode(IntEnum):
    SUCCESS: int
    NO_MATCHING_SUBSCRIBERS: int
    UNSPECIFIED_ERROR: int
    IMPLEMENTATION_SPECIFIC_ERROR: int
    NOT_AUTHORIZED: int
    TOPIC_NAME_INVALID: int
    PACKET_IDENTIFIER_IN_USE: int
    QUOTA_EXCEEDED: int
    PAYLOAD_FORMAT_INVALID: int

class SubackReasonCode(IntEnum):
    GRANTED_QOS_0: int
    GRANTED_QOS_1: int
    GRANTED_QOS_2: int
    UNSPECIFIED_ERROR: int
    IMPLEMENTATION_SPECIFIC_ERROR: int
    NOT_AUTHORIZED: int
    TOPIC_FILTER_INVALID: int
    PACKET_IDENTIFIER_IN_USE: int
    QUOTA_EXCEEDED: int
    SHARED_SUBSCRIPTIONS_NOT_SUPPORTED: int
    SUBSCRIPTION_IDENTIFIERS_NOT_SUPPORTED: int
    WILDCARD_SUBSCRIPTIONS_NOT_SUPPORTED: int

class UnsubackReasonCode(IntEnum):
    SUCCESS: int
    NO_SUBSCRIPTION_EXISTED: int
    UNSPECIFIED_ERROR: int
    IMPLEMENTATION_SPECIFIC_ERROR: int
    NOT_AUTHORIZED: int
    TOPIC_NAME_INVALID: int
    PACKET_IDENTIFIER_IN_USE: int

class ClientSessionBehaviorType(IntEnum):
    DEFAULT: int
    CLEAN: int
    REJOIN_POST_SUCCESS: int
    REJOIN_ALWAYS: int

class PacketType(IntEnum):
    NONE: int
    RESERVED: int
    CONNECT: int
    CONNACK: int
    PUBLISH: int
    PUBACK: int
    PUBREC: int
    PUBREL: int
    PUBCOMP: int
    SUBSCRIBE: int
    SUBACK: int
    UNSUBSCRIBE: int
    UNSUBACK: int
    PINGREQ: int
    PINGRESP: int
    DISCONNECT: int
    AUTH: int

class PayloadFormatIndicator(IntEnum):
    AWS_MQTT5_PFI_BYTES: int
    AWS_MQTT5_PFI_UTF8: int

class RetainAndHandlingType(IntEnum):
    SEND_ON_SUBSCRIBE: int
    SEND_ON_SUBSCRIBE_IF_NEW: int
    DONT_SEND: int

class ExtendedValidationAndFlowControlOptions(IntEnum):
    NONE: int
    AWS_IOT_CORE_DEFAULTS: int

class ClientOperationQueueBehaviorType(IntEnum):
    DEFAULT: int
    FAIL_NON_QOS1_PUBLISH_ON_DISCONNECT: int
    FAIL_QOS0_PUBLISH_ON_DISCONNECT: int
    FAIL_ALL_ON_DISCONNECT: int

class ExponentialBackoffJitterMode(IntEnum):
    DEFAULT: int
    NONE: int
    FULL: int
    DECORRELATED: int

@dataclass
class UserProperty:
    name: Optional[str] = ...
    value: Optional[str] = ...

@dataclass
class NegotiatedSettings:
    maximum_qos: Optional[QoS] = ...
    session_expiry_interval_sec: Optional[int] = ...
    receive_maximum_from_server: Optional[int] = ...
    maximum_packet_size_to_server: Optional[int] = ...
    topic_alias_maximum_to_server: Optional[int] = ...
    topic_alias_maximum_to_client: Optional[int] = ...
    server_keep_alive_sec: Optional[int] = ...
    retain_available: Optional[bool] = ...
    wildcard_subscriptions_available: Optional[bool] = ...
    subscription_identifiers_available: Optional[bool] = ...
    shared_subscriptions_available: Optional[bool] = ...
    rejoined_session: Optional[bool] = ...

@dataclass
class ConnackPacket:
    session_present: Optional[bool] = ...
    reason_code: Optional[ConnectReasonCode] = ...
    session_expiry_interval_sec: Optional[int] = ...
    receive_maximum: Optional[int] = ...
    maximum_qos: Optional[QoS] = ...
    retain_available: Optional[bool] = ...
    maximum_packet_size: Optional[int] = ...
    assigned_client_identifier: Optional[str] = ...
    reason_string: Optional[str] = ...
    user_properties: Optional[Sequence[UserProperty]] = ...
    wildcard_subscriptions_available: Optional[bool] = ...
    subscription_identifiers_available: Optional[bool] = ...
    shared_subscription_available: Optional[bool] = ...
    server_keep_alive: Optional[int] = ...
    response_information: Optional[str] = ...
    server_reference: Optional[str] = ...

@dataclass
class DisconnectPacket:
    reason_code: Optional[DisconnectReasonCode] = ...
    session_expiry_interval_sec: Optional[int] = ...
    reason_string: Optional[str] = ...
    user_properties: Optional[Sequence[UserProperty]] = ...
    server_reference: Optional[str] = ...

@dataclass
class Subscription:
    topic_filter: str
    qos: Optional[QoS] = ...
    no_local: Optional[bool] = ...
    retain_as_published: Optional[bool] = ...
    retain_handling_type: Optional[RetainAndHandlingType] = ...

@dataclass
class SubscribePacket:
    subscriptions: Sequence[Subscription]
    subscription_identifier: Optional[int] = ...
    user_properties: Optional[Sequence[UserProperty]] = ...

@dataclass
class SubackPacket:
    reason_string: Optional[str] = ...
    user_properties: Optional[Sequence[UserProperty]] = ...
    reason_codes: Optional[Sequence[SubackReasonCode]] = ...

@dataclass
class UnsubscribePacket:
    topic_filters: Sequence[str]
    user_properties: Optional[Sequence[UserProperty]] = ...

@dataclass
class UnsubackPacket:
    reason_string: Optional[str] = ...
    user_properties: Optional[Sequence[UserProperty]] = ...
    reason_codes: Optional[Sequence[DisconnectReasonCode]] = ...

@dataclass
class PublishPacket:
    payload: Optional[Any] = ...
    qos: Optional[QoS] = ...
    retain: Optional[bool] = ...
    topic: Optional[str] = ...
    payload_format_indicator: Optional[PayloadFormatIndicator] = ...
    message_expiry_interval_sec: Optional[int] = ...
    topic_alias: Optional[int] = ...
    response_topic: Optional[str] = ...
    correlation_data: Optional[Any] = ...
    subscription_identifiers: Optional[Sequence[int]] = ...
    content_type: Optional[str] = ...
    user_properties: Optional[Sequence[UserProperty]] = ...

@dataclass
class PubackPacket:
    reason_code: Optional[PubackReasonCode] = ...
    reason_string: Optional[str] = ...
    user_properties: Optional[Sequence[UserProperty]] = ...

@dataclass
class ConnectPacket:
    keep_alive_interval_sec: Optional[int] = ...
    client_id: Optional[str] = ...
    username: Optional[str] = ...
    password: Optional[str] = ...
    session_expiry_interval_sec: Optional[int] = ...
    request_response_information: Optional[bool] = ...
    request_problem_information: Optional[bool] = ...
    receive_maximum: Optional[int] = ...
    maximum_packet_size: Optional[int] = ...
    will_delay_interval_sec: Optional[int] = ...
    will: Optional[PublishPacket] = ...
    user_properties: Optional[Sequence[UserProperty]] = ...

class WebsocketHandshakeTransformArgs:
    def __init__(self, client: Client, http_request: HttpRequest, done_future: Future[Any]) -> None:
        self.client: Client
        self.http_request: HttpRequest
    def set_done(self, exception: Optional[Exception] = ...) -> None: ...

@dataclass
class PublishReceivedData:
    publish_packet: Optional[PublishPacket] = ...

@dataclass
class OperationStatisticsData:
    incomplete_operation_count: int = ...
    incomplete_operation_size: int = ...
    unacked_operation_count: int = ...
    unacked_operation_size: int = ...

@dataclass
class LifecycleStoppedData: ...

@dataclass
class LifecycleAttemptingConnectData: ...

@dataclass
class LifecycleConnectSuccessData:
    connack_packet: Optional[ConnackPacket] = ...
    negotiated_settings: Optional[NegotiatedSettings] = ...

@dataclass
class LifecycleConnectFailureData:
    connack_packet: Optional[ConnackPacket] = ...
    exception: Optional[Exception] = ...

@dataclass
class LifecycleDisconnectData:
    disconnect_packet: Optional[DisconnectPacket] = ...
    exception: Optional[Exception] = ...

@dataclass
class PublishCompletionData:
    puback: Optional[PubackPacket] = ...

@dataclass
class ClientOptions:
    host_name: str
    port: Optional[int] = ...
    bootstrap: Optional[ClientBootstrap] = ...
    socket_options: Optional[SocketOptions] = ...
    tls_ctx: Optional[ClientTlsContext] = ...
    http_proxy_options: Optional[HttpProxyOptions] = ...
    websocket_handshake_transform: Optional[Callable[[WebsocketHandshakeTransformArgs], None]] = ...
    connect_options: Optional[ConnectPacket] = ...
    session_behavior: Optional[ClientSessionBehaviorType] = ...
    extended_validation_and_flow_control_options: Optional[
        ExtendedValidationAndFlowControlOptions
    ] = ...
    offline_queue_behavior: Optional[ClientOperationQueueBehaviorType] = ...
    retry_jitter_mode: Optional[ExponentialBackoffJitterMode] = ...
    min_reconnect_delay_ms: Optional[int] = ...
    max_reconnect_delay_ms: Optional[int] = ...
    min_connected_time_to_reset_reconnect_delay_ms: Optional[int] = ...
    ping_timeout_ms: Optional[int] = ...
    connack_timeout_ms: Optional[int] = ...
    ack_timeout_sec: Optional[int] = ...
    on_publish_callback_fn: Optional[Callable[[PublishReceivedData], None]] = ...
    on_lifecycle_event_stopped_fn: Optional[Callable[[LifecycleStoppedData], None]] = ...
    on_lifecycle_event_attempting_connect_fn: Optional[
        Callable[[LifecycleAttemptingConnectData], None]
    ] = ...
    on_lifecycle_event_connection_success_fn: Optional[
        Callable[[LifecycleConnectSuccessData], None]
    ] = ...
    on_lifecycle_event_connection_failure_fn: Optional[
        Callable[[LifecycleConnectFailureData], None]
    ] = ...
    on_lifecycle_event_disconnection_fn: Optional[Callable[[LifecycleDisconnectData], None]] = ...

class _ClientCore:
    def __init__(self, client_options: ClientOptions) -> None: ...

class Client(NativeResource):
    def __init__(self, client_options: ClientOptions) -> None: ...
    def start(self) -> None: ...
    def stop(self, disconnect_packet: Optional[DisconnectPacket] = ...) -> None: ...
    def publish(self, publish_packet: PublishPacket) -> Future[PublishCompletionData]: ...
    def subscribe(self, subscribe_packet: SubscribePacket) -> Future[SubackPacket]: ...
    def unsubscribe(self, unsubscribe_packet: UnsubscribePacket) -> Future[UnsubackPacket]: ...
    def get_stats(self) -> Future[OperationStatisticsData]: ...



from concurrent.futures import Future
from dataclasses import dataclass
from enum import IntEnum
from typing import Any, Callable, Dict, Optional, Tuple, Union

from awscrt import NativeResource as NativeResource
from awscrt.exceptions import AwsCrtError
from awscrt.http import HttpProxyOptions as HttpProxyOptions
from awscrt.http import HttpRequest as HttpRequest
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import ClientTlsContext as ClientTlsContext
from awscrt.io import SocketOptions as SocketOptions

class QoS(IntEnum):
    AT_MOST_ONCE: int
    AT_LEAST_ONCE: int
    EXACTLY_ONCE: int

class ConnectReturnCode(IntEnum):
    ACCEPTED: int
    UNACCEPTABLE_PROTOCOL_VERSION: int
    IDENTIFIER_REJECTED: int
    SERVER_UNAVAILABLE: int
    BAD_USERNAME_OR_PASSWORD: int
    NOT_AUTHORIZED: int

class Will:
    def __init__(self, topic: str, qos: QoS, payload: bytes, retain: bool) -> None:
        self.topic: str
        self.qos: QoS
        self.payload: bytes
        self.retain: bool

class Client(NativeResource):
    def __init__(
        self, bootstrap: Optional[ClientBootstrap] = ..., tls_ctx: Optional[ClientTlsContext] = ...
    ) -> None:
        self.tls_ctx: ClientTlsContext

@dataclass
class OperationStatisticsData:
    incomplete_operation_count: int = ...
    incomplete_operation_size: int = ...
    unacked_operation_count: int = ...
    unacked_operation_size: int = ...

class Connection(NativeResource):
    def __init__(
        self,
        client: Client,
        host_name: str,
        port: int,
        client_id: str,
        clean_session: bool = ...,
        on_connection_interrupted: Optional[Callable[[Connection, AwsCrtError], None]] = ...,
        on_connection_resumed: Optional[
            Callable[[Connection, ConnectReturnCode, bool], None]
        ] = ...,
        reconnect_min_timeout_secs: int = ...,
        reconnect_max_timeout_secs: int = ...,
        keep_alive_secs: int = ...,
        ping_timeout_ms: int = ...,
        protocol_operation_timeout_ms: int = ...,
        will: Optional[Will] = ...,
        username: Optional[str] = ...,
        password: Optional[str] = ...,
        socket_options: Optional[SocketOptions] = ...,
        use_websockets: bool = ...,
        websocket_proxy_options: Optional[HttpProxyOptions] = ...,
        websocket_handshake_transform: Optional[
            Callable[[WebsocketHandshakeTransformArgs], None]
        ] = ...,
        proxy_options: Optional[HttpProxyOptions] = ...,
    ) -> None:
        self.client: Client
        self.client_id: str
        self.host_name: str
        self.port: int
        self.clean_session: bool
        self.reconnect_min_timeout_secs: int
        self.reconnect_max_timeout_secs: int
        self.keep_alive_secs: int
        self.ping_timeout_ms: int
        self.protocol_operation_timeout_ms: int
        self.will: Will
        self.username: str
        self.password: str
        self.socket_options: Optional[SocketOptions]
        self.proxy_options: Optional[HttpProxyOptions]
    def connect(self) -> Future[Optional[BaseException]]: ...
    def reconnect(self) -> Future[Optional[BaseException]]: ...
    def disconnect(self) -> Future[Optional[BaseException]]: ...
    def subscribe(
        self,
        topic: str,
        qos: QoS,
        callback: Optional[Callable[[str, bytes, bool, QoS, bool], None]] = ...,
    ) -> Tuple[Future[Optional[Dict[str, Any]]], int]: ...
    def on_message(self, callback: Callable[[str, bytes, bool, QoS, bool], None]) -> None: ...
    def unsubscribe(self, topic: str) -> Tuple[Future[Optional[Dict[str, Any]]], int]: ...
    def resubscribe_existing_topics(self) -> Tuple[Future[Optional[Dict[str, Any]]], int]: ...
    def publish(
        self, topic: str, payload: Union[str, bytes, bytearray], qos: QoS, retain: bool = ...
    ) -> Tuple[Future[Optional[Dict[str, Any]]], int]: ...
    def get_stats(self) -> OperationStatisticsData: ...

class WebsocketHandshakeTransformArgs:
    def __init__(
        self,
        mqtt_connection: Connection,
        http_request: HttpRequest,
        done_future: Future[Optional[BaseException]],
    ) -> None:
        self.mqtt_connection: Connection
        self.http_request: HttpRequest
    def set_done(self, exception: Optional[BaseException] = ...) -> None: ...

class SubscribeError(Exception): ...
